;TODOs
; ? use ball for sword(s), flicker?
; ? only one player can display sword (saves 5 cycles)
; - specialized kernels
; - 2.5D effects
; ? player repositioning
; - minimize RAM consumption
; - gates
; o eye candy (torches, windows, bricks, pieces)
; - swap GRP0, GRP1 usage (Prince = GRP0!)

; Probleme:
; - GPR1 liegt hinter GRP0
; - BL wird durch GRP1 VDELED

TIA_BASE_READ_ADDRESS = $30

    processor 6502
    include vcs.h

;===============================================================================
; A S S E M B L E R - S W I T C H E S
;===============================================================================

NTSC        = 1
ILLEGAL     = 1
DEBUG       = 1


;===============================================================================
; C O N S T A N T S
;===============================================================================
; Original:
; - screen: 320 x 189
; - Prince: 41
; Atari 2600: (factor: 2*1.25)
; - screen: 128 x  75
; - Prince: 16

VERSION     = $0000
BASE_ADR    = $f000

NUM_FLOORS  = 3
NUM_PARTS   = 3
NUM_BLOCKS  = NUM_FLOORS*NUM_PARTS+2

FLOOR_H     = 25
KERNEL_H    = FLOOR_H*NUM_PARTS+2

PLAYER_H    = 10    ; should be ~16+
ENEMY_H     = 18;+8

COL_BK      = $00


;===============================================================================
; Z P - V A R I A B L E S
;===============================================================================

    SEG.U   variables
    ORG     $80

frameCnt    .byte
tmpVar      ds 10

colLst      ds NUM_PARTS

pf1LLst     ds NUM_BLOCKS
pf2LLst     ds NUM_BLOCKS
pf2RLst     ds NUM_BLOCKS
pf1RLst     ds NUM_BLOCKS
; walls:
pf1LLst1    = pf1LLst
pf2LLst1    = pf2LLst
pf2RLst1    = pf2RLst
pf1RLst1    = pf1RLst
; bottom:
pf1LLst2    = pf1LLst + NUM_FLOORS
pf2LLst2    = pf2LLst + NUM_FLOORS
pf2RLst2    = pf2RLst + NUM_FLOORS
pf1RLst2    = pf1RLst + NUM_FLOORS
; ceiling:
pf1LLst3    = pf1LLst + NUM_FLOORS*2 + 1
pf2LLst3    = pf2LLst + NUM_FLOORS*2 + 1
pf2RLst3    = pf2RLst + NUM_FLOORS*2 + 1
pf1RLst3    = pf1RLst + NUM_FLOORS*2 + 1


xLst        ds 2
xP0         = xLst
xP1         = xLst+1

yLst        ds 2
yP0         = yLst
;yP1         = yLst+1
yBL         .byte
hBL         .byte

ptrP0       .word
ptrC0       .word
ptrBL       .word

gateDir     .byte

xPosP0Lst   ds NUM_FLOORS
dummy       .byte
hmPosP1Lst  ds NUM_FLOORS
yPosP1Lst   ds NUM_FLOORS
ptrP1Lst    ds NUM_FLOORS
ptrC1Lst    ds NUM_FLOORS
ptrHiLst    ds NUM_FLOORS
nusizLst    ds NUM_FLOORS


  ECHO "*** RAM: ", ., " ***"


;===============================================================================
; M A C R O S
;===============================================================================

DEBUG_BYTES SET 0

  MAC DEBUG_BRK
    IF DEBUG
DEBUG_BYTES SET DEBUG_BYTES + 1
      brk                         ;
    ENDIF
  ENDM

  MAC BIT_B
    .byte   $24
  ENDM

  MAC BIT_W
    .byte   $2c
  ENDM

  MAC SLEEP
    IF {1} = 1
      ECHO "ERROR: SLEEP 1 not allowed !"
      END
    ENDIF
    IF {1} & 1
      nop $00
      REPEAT ({1}-3)/2
        nop
      REPEND
    ELSE
      REPEAT ({1})/2
        nop
      REPEND
    ENDIF
  ENDM

  MAC CHECKPAGE
    IF >. != >{1}
      ECHO ""
      ECHO "ERROR: different pages! (", {1}, ",", ., ")"
      ECHO ""
      ERR
    ENDIF
  ENDM

  MAC ALIGN_OFS
.Y SET .
    org ((. + $100 - {1}) & $ff00) + {1}
.Y SET . - .Y
  ECHO "ALIGN_OFS:", .Y
  ENDM


;===============================================================================
; R O M - C O D E
;===============================================================================
    SEG     Bank0
    ORG     BASE_ADR

Start SUBROUTINE
    cld                         ;           Clear BCD math bit.
    lda     #0
    txs
.clearLoop:
    tsx
    pha
    bne     .clearLoop

    jsr     GameInit

.mainLoop:
    jsr     VerticalBlank
    jsr     GameCalc
    jsr     DrawScreen
    jsr     OverScan
    jmp     .mainLoop


GameInit SUBROUTINE
    lda     #COL_BK
    sta     COLUBK

    lda     #1
    sta     VDELP0
    sta     VDELBL

    ldx     #NUM_BLOCKS-1
.loopInit
    lda     PF1LTbl,x
    sta     pf1LLst,x
    lda     PF2LTbl,x
    sta     pf2LLst,x
    lda     PF1RTbl,x
    sta     pf1RLst,x
    lda     PF2RTbl,x
    sta     pf2RLst,x
    dex
    bpl     .loopInit

    ldx     #2
.loopInitCol:
    lda     ColorTbl,x
    sta     colLst,x
    dex
    bpl     .loopInitCol

    lda     #66
    sta     xP0
    lda     #89
    sta     xP1

    lda     #KERNEL_H-FLOOR_H+PLAYER_H
;    lda     #82
    sta     yP0
;    lda     #82

;    lda     #$80
;    sta     hBL
;    lda     #KERNEL_H-1-2
;    sta     yBL

    rts
; GameInit


VerticalBlank SUBROUTINE
    lda     #%00001110
.loopVSync:
    sta     WSYNC
    sta     VSYNC
    lsr
    bne     .loopVSync

    inc     frameCnt
  IF NTSC
    lda     #44
  ELSE
    lda     #77
  ENDIF
    sta     TIM64T
    rts
; VerticalBlank


GameCalc SUBROUTINE
    lda     #<Prince0+PLAYER_H
    sec
    sbc     yP0
    sta     ptrP0
    lda     #<Color0+PLAYER_H
;    sec
    sbc     yP0
    sta     ptrC0

    ldx     #2
.loopXPos:
    lda     XPosTbl,x
    sta     xPosP0Lst,x
    dex
    bpl     .loopXPos
    lda     xP1
    sta     xPosP0Lst;+2

    ldx     #2
.loopPtrs:
    lda     YPosTbl,x
    sta     yPosP1Lst,x

    lda     xPosP0Lst,x
    jsr     CalcXPos
    tay
    lda     tmpVar
    sty     tmpVar
    cmp     #5
    bcc     .left
    sbc     #5
.left:
    rol
    ora     tmpVar
    sta     hmPosP1Lst,x

    lda     PtrGraphics,x
    clc
    adc     #ENEMY_H
    sec
    sbc     yPosP1Lst,x
    sta     ptrP1Lst,x
    lda     PtrColors,x
    clc
    adc     #ENEMY_H
    sec
    sbc     yPosP1Lst,x
    sta     ptrC1Lst,x
    lda     PtrHi,x
    sta     ptrHiLst,x

    lda     NusizTbl,x
    sta     nusizLst,x
    dex
    bpl     .loopPtrs

    lda     yPosP1Lst+2
    sec
    sbc     #ENEMY_H|$80
    sta     yPosP1Lst+2
;
;    lda     #<Prince0+ENEMY_H
;;    sec
;    sbc     yPosP1Lst+2
;    sta     ptrP1Lst+2
;    lda     #<EnemyCol0+ENEMY_H
;;    sec
;    sbc     yPosP1Lst+2
;    sta     ptrC1Lst+2

;    lda     hBL
;    and     #$7f
;    clc
;    adc     #<GateBL
;    sec
;    sbc     yBL
;    sta     ptrBL

;    lda     #8
;    sta     REFP0

;    lda     #45

; position ball:
;    ldx     #4
;    lda     #18
;    jsr     SetXPos
;---------------------------------------
; position enemy missile
    lda     frameCnt
    lsr
    bcc     .princeSword

    lda     #0
    sta     tmpVar+1
    lda     #KERNEL_H-1
    sec
    sbc     yPosP1Lst;+2
    asl
    rol     tmpVar+1
    clc
    adc     xPosP0Lst;+2
    bcc     .skipInc1a
    inc     tmpVar+1
.skipInc1a:
    clc
    adc     #31+16
    bcc     .skipInc1b
    inc     tmpVar+1
.skipInc1b:
    sta     tmpVar
.loopM1:
    lda     tmpVar
    sec
    sbc     #160
    sta     tmpVar
    lda     tmpVar+1
    sbc     #0
    sta     tmpVar+1
    bcs     .loopM1

    lda     tmpVar
    adc     #160
    jmp     .posSword
;---------------------------------------
.princeSword:
; position Prince missile
    lda     #0
    sta     tmpVar+1
    lda     #KERNEL_H-1
    sec
    sbc     yP0
    asl
    rol     tmpVar+1
    asl
    rol     tmpVar+1
    clc
    adc     #27+1
    bcc     .skipInc0
    inc     tmpVar+1
.skipInc0:
    sta     tmpVar

    lda     xP0
    sec
    sbc     tmpVar
    sta     tmpVar
    lda     #0
    sbc     tmpVar+1
    sta     tmpVar+1

.loopM0:
    lda     tmpVar
    clc
    adc     #160
    sta     tmpVar
    lda     tmpVar+1
    adc     #0
    sta     tmpVar+1
    bmi     .loopM0
    lda     tmpVar
    cmp     #160
    bcc     .skipSub0
    sbc     #160
.skipSub0:

.posSword:
    ldx     #4
    jsr     SetXPos

    ldx     #3
    lda     xPosP0Lst
    clc
    adc     #17
    jsr     SetXPos
    dex
    lda     xP0
    jsr     SetXPos

    ldx     #2
;---------------------------------------
    dex
;    lda     xP1
;    jsr     SetXPos
    dex
    lda     xP0
    jsr     SetXPos

    sta     WSYNC
    sta     HMOVE
    sta     WSYNC
    sta     HMCLR

    lda     #%010101    ;%100101
    sta     NUSIZ0
    lda     #%010101
    sta     NUSIZ1


;    lda     #$c0
;    sta     HMM0
;    lda     #$20
;    sta     HMM1

    lda     frameCnt
    lsr
    bcs     .enemySword

    lda     yP0
    sec
    sbc     #2
    sta     yBL
    lda     #$03|$80
    sta     hBL

    ldx     #$c0
    ldy     #%100001
    bne     .contSword

.enemySword
    lda     yPosP1Lst
    sec
    sbc     #8
    sta     yBL
    lda     #$05|$80
    sta     hBL
    ldx     #$20
    ldy     #%010001
.contSword:
    stx     HMBL
    sty     CTRLPF

    rts
; GameCalc


DrawScreen SUBROUTINE
; Kernel:
; Main: both players
; Ground1
; Groudn2:
; Repos:

;.floor  = tmpVar+1
.yP1    = tmpVar+2
.ptrP1  = tmpVar+4
.ptrC1  = tmpVar+6


    lda     #>Prince0
    sta     ptrP0+1
    lda     #>Color0
    sta     ptrC0+1

;    lda     #>Prince0
;    sta     .ptrP0+1
;    lda     #>EnemyCol0
;    sta     .ptrC0+1

    lda     #>GateBL
    sta     ptrBL+1

;    ldx     #NUM_FLOORS
;    stx     .floor

    ldx     #227
.waitTim:
    lda     INTIM
    bne     .waitTim
    sta     WSYNC
;---------------------------------------------------------------
    sta     VBLANK          ; 3
    stx     TIM64T          ; 4

    SLEEP   58-18

;    ldx     #NUM_BLOCKS     ; 2
    ldx     #NUM_FLOORS
    txs                     ; 2
    ldy     #KERNEL_H-1     ; 2
    tya                     ; 2
    jmp     .enterKernel    ; 3 = 16

;***********************************************************
    align   256

.switchBL:                  ;           @69
; ball is NOT drawn:
;    tya                     ; 2
    sbc     hBL             ; 3
    sta.w   yBL             ; 3
    bcc     .contBL         ; 3 =  9    @02
    DEBUG_BRK

.waitBL:                    ;           @71
    lda     #0
    sta     ENABL
;    SLEEP   4               ; 4
    beq     .contBL         ; 3         @02

;===========================================================
.waitP0                     ;           @35
    SLEEP   3               ; 3
    bne     .contWait0      ; 3 =  6            ;

.switchP0:                  ;           @31
; player 0 is NOT drawn:
    bne     .waitP0         ; 2
    tya                     ; 2
    sbc     #PLAYER_H|$80   ; 2
    sta     yP0             ; 3 =  9    @41
.contWait0:
    lda     pf2RLst1,x      ; 4
    sta.w   PF2             ; 3         @48 *!*
    lda     pf1RLst1,x      ; 4
    sta     PF1             ; 3 = 14    @55

    cpy     .yP1            ; 3
    bpl     .switchP01      ; 2=  5/6
; player one is drawn
    lax     (.ptrP1),y      ; 5
    lda     #0              ; 2
    sta     GRP0            ; 3         @70
    SLEEP   2               ; 2                 ENAM0???
    beq     .contDraw0      ; 3 = 15    @75

.switchP01:                 ;           @61
; both players are NOT drawn:
    bne     .waitP01        ; 2
    tya                     ; 2
    sbc     #ENEMY_H|$80    ; 2
    sta     .yP1            ; 3 =  9
.contWait01:                ;           @70
    lda     #0              ; 2
    sta     GRP0            ; 3 =  5    @75
;---------------------------------------
    sta.w   HMOVE           ; 4         @03
    sta     ENAM0           ; 3
    SLEEP   3               ; 3
    sta     GRP1            ; 3
    beq     .contDraw1      ; 3 = 16    @15     ;

.waitP01:                   ;           @64
    SLEEP   3               ; 3
    bne     .contWait01     ; 3 =  6

;===========================================================
.waitP1:                    ;           @54
    SLEEP   3               ; 3
    bne     .contWait1      ; 3 =  6           ;

.switchP1:                  ;           @51
; player 0 is drawn, player 1 is NOT drawn:
    lda     pf1RLst1,x      ; 4
    sta     PF1             ; 3 =  7    @58
    cpy     .yP1            ; 3
    bne     .waitP1         ; 2
    tya                     ; 2
    sbc     #ENEMY_H|$80    ; 2
    sta     .yP1            ; 3 = 12    @70
.contWait1:

    lda     (ptrC0),y       ; 5
    sta     COLUP0          ; 3 =  8    @02
;---------------------------------------
    sta     HMOVE           ; 3         @05
    asl                     ; 2
    sta     ENAM0           ; 3 =  8    @10

    lda     #0              ; 2
    sta     GRP1            ; 3                 ENAM1???
    beq     .contDraw1n     ; 3 = 10    @20

;===========================================================
.nextRow:                   ;           @59
    SLEEP   4               ; 4 =  4
.nextRow5:                   ;          @63
    cpy     yBL             ; 3
    beq     .switchBL       ; 2
    bpl     .waitBL         ; 2
    lda     #2              ; 2
    SLEEP   4               ; 4
;---------------------------------------
    sta     ENABL           ; 3 = 16    @VDELed!
.contBL:
    lda     colLst+2        ; 3
    sta     COLUPF          ; 3 =  6    @09

    lda     pf1LLst1,x      ; 4
    sta     PF1             ; 3         @16
    lda     pf2LLst1,x      ; 4
    sta     PF2             ; 3 = 14    @23

    txs                     ; 2
    cpy     yP0             ; 3
    bpl     .switchP0       ; 2=  7/8

    lda     (ptrP0),y       ; 5
    sta     GRP0            ; 3 =  8    @VDELed!

    lda     pf2RLst1,x      ; 4
    cpy     .yP1            ; 3
    sta     PF2             ; 3         @48 *!*
    bpl     .switchP1       ; 2= 19/20
    lda     pf1RLst1,x      ; 4
    sta     PF1             ; 3 =  7    @57

    lax     (.ptrP1),y      ; 5 =  5

    lda     (ptrC0),y       ; 5
    sta     COLUP0          ; 3         @70
    asl                     ; 2
    sta     ENAM0           ; 3 = 13    @75
.contDraw0:
;---------------------------------------
    sta     HMOVE           ; 3         @02
    stx     GRP1            ; 3         @05
    lda     (.ptrC1),y      ; 5 = 11
    sta     COLUP1          ; 3         @13
    asl                     ; 2
.contDraw1:                 ;           @15
    sta     ENAM1           ; 3 =  8    @18
.contDraw1n:                ;           @20
    tsx                     ; 2
    lda     pf1LLst1,x      ; 4
    sta     PF1             ; 3         @27
    lda     pf2LLst1,x      ; 4
    sta     PF2             ; 3         @34
    lda     pf1RLst1,x      ; 4
    sta     PF1             ; 3         @41
    lda     pf2RLst1,x      ; 4
    sta     PF2             ; 3 = 30    @48 *!*

    dey                     ; 2
    tya                     ; 2
    cmp     BlockTbl,x      ; 4
    bne     .nextRow        ; 2= 10/11

; *** bottom floor; ***
.enterKernel:               ;           @58
    lda     ptrP1Lst-1,x    ; 4
    sta     .ptrP1          ; 3
    lda     yPosP1Lst-1,x   ; 4
    sta     .yP1            ; 3
    lda     ptrC1Lst-1,x    ; 4 = 18    @76
;---------------------------------------
    sta     .ptrC1          ; 3
    lda     ptrHiLst-1,x    ; 4
    sta     .ptrP1+1        ; 3
    sta     .ptrC1+1        ; 3 = 13

    lda     colLst+1        ; 3
    sta.w   COLUPF          ; 4 =  6    @20

    lda     pf1LLst3,x      ; 4
    sta     PF1             ; 3         @27
    lda     pf2LLst3,x      ; 4
    sta     PF2             ; 3         @34
    lda     pf1RLst3,x      ; 4
    sta     PF1             ; 3         @41
    lda     pf2RLst3,x      ; 4
    sta     PF2             ; 3 = 28    @48 *!*

    SLEEP   9               ; 9 =  9

    tya                     ; 2
    cpy     yP0             ; 3
    bpl     .switchP0a      ; 2
    lda     (ptrP0),y       ; 5 = 12
    sta     GRP0            ; 3         @07
    lda     (ptrC0),y       ; 5
;---------------------------------------
    sta     HMOVE           ; 3         @04
    sta     COLUP0          ; 3         @07
    asl                     ; 2 =  8
.contDraw1a:                ;           @09
    sta     ENAM0           ; 3         @12
    lda     #0              ; 2
    sta     GRP1            ; 3         @17
    sta     ENAM1           ; 3 = 11    @20

    lda     pf1LLst3,x      ; 4
    sta     PF1             ; 3         @27
    lda     pf2LLst3,x      ; 4
    sta     PF2             ; 3         @34
    lda     pf1RLst3,x      ; 4
    sta     PF1             ; 3         @41
    lda     pf2RLst3,x      ; 4
    sta     PF2             ; 3 = 28    @48 *!*

    dey                     ; 2 =  2

    SLEEP   6               ; 6 =  6

    lda     nusizLst-1,x    ; 4
    sta     NUSIZ1          ; 3
    sta     REFP1           ; 3 = 10

    lda     colLst          ; 3
    sta     COLUPF          ; 3 =  6    @70

    lda     #0              ; 2
    sta     PF2             ; 3         @75
;---------------------------------------
    sta     PF1             ; 3 =  8    @02
; *** blank, repositioning floor ***
    lda     hmPosP1Lst-1,x  ; 4
    sta     HMP1            ; 3
    and     #$0f            ; 2
    lsr                     ; 2
    bcs     .right          ; 2= 15/16
    beq     .pos0l          ; 2
    sec                     ; 2
.loopWaitLeft:
    sbc     #1              ; 2
    bne     .loopWaitLeft   ; 2
.pos0l:
    sta     RESP1           ; 3 =  6    @23..43

;    SLEEP   9

    cpy     yP0             ; 3
    bpl     .switchP0l      ; 2
    lda     (ptrP0),y       ; 5
    sta     GRP0            ; 3
    lda     (ptrC0),y       ; 5
    jmp     .wsync          ; 3 = 21

;===========================================================
.switchP0a:                 ;           @66
    bne     .waitP0a        ; 2
    sbc     #PLAYER_H|$80   ; 2
    sta     yP0             ; 3
    lda     #0              ; 2
;---------------------------------------
    sta     GRP0            ; 3         @02
    sta.w   HMOVE           ; 3         @05
    beq     .contDraw1a     ; 4 =  9    @09

.waitP0a:                   ;           @69
    SLEEP   4               ; 4
    lda     #0              ; 2
;---------------------------------------
    sta     GRP0            ; 3         @02
    sta.w   HMOVE           ; 3         @03
    beq     .contDraw1a     ; 4 = 10

;===========================================================
.right:                     ;           @18
    beq     .pos0r          ; 2
    nop                     ; 2
.loopWaitRight:
    sbc     #1              ; 2
    bne     .loopWaitRight  ; 2= 3..28
.pos0r:

    SLEEP   6               ; 6

    cpy     yP0             ; 3
    bpl     .switchP0r      ; 2
    lda     (ptrP0),y       ; 5
    sta     GRP0            ; 3
    lda     (ptrC0),y       ; 5
    sta     RESP1           ; 3 = 21    @48..73
.wsync:
    sta     WSYNC           ;
;---------------------------------------
    sta     HMOVE           ; 3         @03
    sta.w   COLUP0          ; 3         @06
    asl                     ; 3 =  9
.contDraw1b:                ;           @09
    dey                     ; 2
    sta     ENAM0           ; 3
    lda     #0              ; 2
    sta.w   GRP1            ; 4 = 11    @20

    lda     pf1LLst2,x      ; 4
    sta     PF1             ; 3         @27
    lda     pf2LLst2,x      ; 4
    sta     PF2             ; 3         @34
    lda     pf1RLst2,x      ; 4
    sta     PF1             ; 3         @41
    lda     pf2RLst2,x      ; 4
    sta     PF2             ; 3 = 28    @48 *!*

    lda     #0              ; 2
    sta.w   HMP1            ; 4 =  6

    dex                     ; 2
    bmi     .exitLoop       ; 2
    tya                     ; 2
    jmp     .nextRow5       ; 3 =  9

;===========================================================
.switchP0l:                 ;
    bne     .waitP0l        ; 2
    tya                     ; 2
    sbc     #PLAYER_H|$80   ; 2
    sta     yP0             ; 3 =  8
.waitP0l:
    lda     #0              ; 2
    sta     WSYNC           ; 3
;---------------------------------------
    sta     HMOVE           ; 3         @03
    sta     GRP0            ; 3
    beq     .contDraw1b     ; 3 =  9    @09

;===========================================================
.switchP0r:                 ;           @33+n*5
    bne     .waitP0r        ; 2
    tya                     ; 2
    sbc     #PLAYER_H|$80   ; 2
    sta     yP0             ; 3
.contWait0r:                ;           @42
    lda     #0              ; 2
    sta.w   RESP1           ; 4 = 15    @48+n*5
    sta     WSYNC           ; 3
;---------------------------------------
    sta     HMOVE           ; 3         @03
    sta     GRP0            ; 3
    beq     .contDraw1b     ; 3 =  9    @09

.waitP0r:                   ;           @36
    SLEEP   3               ; 3
    bne     .contWait0r     ; 3 =  6    @42

.exitLoop:
    sta     WSYNC
;---------------------------------------------------------------
;    stx     VBLANK
    ldy     #0
    sty     PF1
    sty     PF2
    sty     ENAM1
    sty     ENAM0
    sty     ENABL
    sty     GRP0
    sty     GRP1

    sta     RESP1
    sta     RESP0
    lda     #$f0
    sta     HMP1

    sty     REFP0
    sty     REFP1

    lda     #$44
    sta     COLUP1
    sta     COLUP0

    lda     #%000000
    sta     NUSIZ0
    lda     #%000000
    sta     NUSIZ1

    sta     WSYNC
    sta     HMOVE
;    sty     VBLANK
    ldx     #4
.loopLives
    sta     WSYNC
    lda     LivesTbl1,x
    sta     GRP0
    lda     LivesTbl0,x
    sta     GRP1
    dex
    bpl     .loopLives
    sta     WSYNC

    stx     VBLANK
    sty     GRP0
    sty     GRP1
    sty     COLUPF

    ldx     #(92-KERNEL_H)*2-1
.loopBottom:
    dex
    sta     WSYNC
    bne     .loopBottom

    ldx     #$fd
    txs
    rts
; DrawScreen


OverScan SUBROUTINE
  IF NTSC
    lda     #36
  ELSE
    lda     #63
  ENDIF
    sta     TIM64T

;    lda     yBL             ; 3
;    clc
;    adc     hBL
;    sta     yBL

;    lda     yP0             ; 3
;    clc
;    adc     #ENEMY_H|$80
;    sta     yP0

    lda     yP0            ; 3
    clc
    adc     #PLAYER_H|$80
    sta     yP0

    ldx     #1
    bit     INPT4
    bpl     .skipInx
    dex
.skipInx:
    ldy     xLst,x
    lda     SWCHA
    bmi     .skipRight
    cpy     #159
    bcs     .skipRight
    iny
.skipRight
    asl
    bmi     .skipLeft
    cpy     #1
    bcc     .skipLeft
    dey
.skipLeft:
    sty     xLst,x

    ldy     yLst,x
    cpx     #1
    beq     .skipDown
    asl
    bmi     .skipUp
    cpy     #1
    bcc     .skipUp
    dey
.skipUp:
    asl
    bmi     .skipDown
    cpy     #KERNEL_H-1
    bcs     .skipDown
    iny
.skipDown:
    sty     yLst,x

;    lda     hBL
;    and     #$7f
;    tax
;    lda     frameCnt
;    and     #$0f
;    beq     .moveGate
;    and     #$08
;    lsr
;    lsr
;    lsr
;    bit     gateDir
;    bpl     .down
;    eor     #$01
;.down:
;    sta     VDELBL
;    bpl     .skipGate
;
;.moveGate:
;    bit     gateDir
;    bmi     .closing
;    BIT_W
;.startOpening
;    asl     gateDir
;    cpx     #FLOOR_H-3
;    bcs     .startClosing
;    inc     hBL
;    jmp     .skipGate
;
;.startClosing:
;    ror     gateDir
;.closing:
;    cpx     #1
;    bcc     .startOpening
;    dec     hBL
;.skipGate:
;    lda     hBL
;    ora     #$80
;    sta     hBL

.waitTim:
    lda     INTIM
    bne     .waitTim
    rts
; OverScan

    align 256

;***********************************************************
CalcXPos SUBROUTINE
; 48-55 cycles (avg.:~51)
; Input : a (= xpos)
; Output: a, tmpVar (= HMOVE)
; Uses  : a, y, tmpVar
;***********************************************************
;   0 -> -4 /  0
; 159 -> +5 / 10
;    cmp     #12                 ; 2
;    bcc     .skipAdd            ; 2
;    adc     #2                  ; 2         3 pixel correction (C=1!)
;CalcMXPos:
;.skipAdd:                       ;   = 5/6
    tay                         ; 2
    lsr                         ; 2
    lsr                         ; 2
    lsr                         ; 2
    lsr                         ; 2
    sta     tmpVar              ; 3 = 13
    tya                         ; 2
    and     #$0f                ; 2
    clc                         ; 2
    adc     tmpVar              ; 3
    cmp     #$0f                ; 2
    bcc     .nextPos            ; 2
    sbc     #$0f                ; 2
    inc     tmpVar              ; 5
.nextPos:                       ;   = 14/20
    eor     #%00000111          ; 2
    asl                         ; 2
    asl                         ; 2
    asl                         ; 2
    asl                         ; 2
;    ora     tmpVar              ; 3         lo-value in tmpVar
    rts                         ; 6 = 16


SetXPos SUBROUTINE
    sec
    sta     WSYNC
WaitObject:
    sbc     #$0f            ; 2
    bcs     WaitObject      ; 2

  CHECKPAGE WaitObject

    eor     #$07            ; 2
    asl                     ; 2
    asl                     ; 2
    asl                     ; 2
    asl                     ; 2
    sta     HMP0,x          ; 4
    sta.wx  RESP0,x         ; 5     @23!
    rts
; SetXPos


;===============================================================================
; R O M - T A B L E S (Bank 0)
;===============================================================================
    org     BASE_ADR + $600

;BlockTbl:
;    .byte                         KERNEL_H-FLOOR_H*3-2, KERNEL_H-FLOOR_H*3-1, KERNEL_H-FLOOR_H*3
;    .byte   KERNEL_H-FLOOR_H*2-3, KERNEL_H-FLOOR_H*2-2, KERNEL_H-FLOOR_H*2-1, KERNEL_H-FLOOR_H*2
;    .byte   KERNEL_H-FLOOR_H*1-3, KERNEL_H-FLOOR_H*1-2, KERNEL_H-FLOOR_H*1-1, KERNEL_H-FLOOR_H*1
;    .byte   KERNEL_H-FLOOR_H*0-3, KERNEL_H-FLOOR_H*0-2
;ColorTbl:
;    .byte        $98, $9e, $94
;    .byte   $90, $98, $9e, $94
;    .byte   $90, $98, $9e, $94
;    .byte   $90, $98
;
;PF1LTbl:
;    .byte        $ff, $00, $ff
;    .byte   $ff, $f1, $7f, $80
;    .byte   $80, $80, $80, $00
;    .byte   $ff, $ff
;PF2LTbl:
;    .byte        $ff, $e0, $1f
;    .byte   $1f, $1f, $1f, $08
;    .byte   $08, $fc, $fc, $00
;    .byte   $ff, $ff
;PF2RTbl:
;    .byte        $ff, $ff, $40
;    .byte   $40, $40, $00, $ff
;    .byte   $ff, $ff, $ff, $00
;    .byte   $ff, $ff
;PF1RTbl:
;    .byte        $ff, $0f, $f0
;    .byte   $f0, $f0, $00, $ff
;    .byte   $ff, $ff, $01, $fe
;    .byte   $ff, $ff

;BlockTbl:
;    .byte   KERNEL_H-FLOOR_H*3-3, KERNEL_H-FLOOR_H*3-2, KERNEL_H-FLOOR_H*3-1
;    .byte   KERNEL_H-FLOOR_H*2-3, KERNEL_H-FLOOR_H*2-2, KERNEL_H-FLOOR_H*2-1
;    .byte   KERNEL_H-FLOOR_H*1-3, KERNEL_H-FLOOR_H*1-2, KERNEL_H-FLOOR_H*1-1
;    .byte   KERNEL_H-FLOOR_H*0-3, KERNEL_H-FLOOR_H*0-2
BlockTbl:
    .byte   KERNEL_H-FLOOR_H*3-1
    .byte   KERNEL_H-FLOOR_H*2-1
    .byte   KERNEL_H-FLOOR_H*1-1

ColorTbl:
    .byte   $98, $9a, $94

PF1LTbl:
    .byte   $ff, $80, $00
    .byte   $ff, $f1, $80, $80
    .byte   $ff, $ff, $80, $ff
PF2LTbl:
    .byte   $1f, $08, $00
    .byte   $ff, $1f, $08, $00
    .byte   $ff, $1f, $fc, $ff
PF2RTbl:
    .byte   $40, $ff, $00

    .byte   $ff, $40, $ff, $00
    .byte   $fc, $40, $ff, $ff
PF1RTbl:
    .byte   $f0, $ff, $fe

    .byte   $ff, $f0, $ff, $fe
    .byte   $fe, $f0, $ff, $ff

    ALIGN_OFS KERNEL_H

Prince0:
    .byte   %00100001
    .byte   %01100110
    .byte   %01111110
    .byte   %00111000
    .byte   %00011000
    .byte   %11011000
    .byte   %00110000
    .byte   %01101110
    .byte   %01100000
    .byte   0
    .byte   0
    .byte   0
    .byte   0
    .byte   0
    .byte   0
    .byte   0
    .byte   0
    .byte   0
Color0:
    .byte   $4a
    .byte   $0e
    .byte   $0e
    .byte   $0e
    .byte   $4a
    .byte   $4a|1
    .byte   $4a
    .byte   $4a
    .byte   $4a
    .byte   $00
    .byte   $00

Prince1:
    .byte   %00110011
    .byte   %00010001
    .byte   %00110111
    .byte   %00110111
    .byte   %00111110
    .byte   %00011110
    .byte   %00011110
    .byte   %00001110
    .byte   %00001101
    .byte   %00001101
    .byte   %00111111
    .byte   %00001111
    .byte   %00001110
    .byte   %00000010
    .byte   %00001110
    .byte   %00001110
    .byte   %00001110
    .byte   %00011110
Color1:
    .byte   $44
    .byte   $40
    .byte   $06
    .byte   $0a
    .byte   $0e
    .byte   $0e
    .byte   $0a
    .byte   $40
    .byte   $44
    .byte   $48
    .byte   $4a
    .byte   $4a
    .byte   $4a
    .byte   $44
    .byte   $4a
    .byte   $4a
    .byte   $28
    .byte   $14

;Enemy0:
;    .byte   %00100100
;    .byte   %00101100
;    .byte   %00101100
;    .byte   %00111000
;    .byte   %00111000
;    .byte   %00110100
;    .byte   %00110100
;    .byte   %00111000
;    .byte   %00010000
;    .byte   %00110000
;    .byte   %00110000
EnemyCol0:
    .byte   $28
    .byte   $28
    .byte   $42
    .byte   $42
    .byte   $aa
    .byte   $aa|1
    .byte   $aa
    .byte   $4a
    .byte   $42
;    .byte   $00
;    .byte   $00
;    .byte   $00
;    .byte   $00
;    .byte   $00
;    .byte   $00
;    .byte   $00
;    .byte   $00
;    .byte   $00

Torch0:
    .byte   0
    .byte   %00000100
    .byte   %00001100
;    .byte   %00001100
    .byte   %00001100
    .byte   %00011100
    .byte   %00011100
    .byte   %00011100
    .byte   %00111110
    .byte   %00011000
    .byte   %00001100
    .byte   %00011100
    .byte   %00011110
    .byte   %00011100
    .byte   %00111000
    .byte   %00011100
    .byte   %00001000
    .byte   0
    .byte   0
TorchCol0:
    .byte   $04
    .byte   $06
    .byte   $08
    .byte   $08
    .byte   $08
    .byte   $08
    .byte   $02
    .byte   $0c
    .byte   $1c
    .byte   $2c
    .byte   $2a
    .byte   $2c
    .byte   $3c
    .byte   $3a
    .byte   $3c
    .byte   $38
;    .byte   0
;    .byte   0
Bricks:
    .byte   %00000000
    .byte   %00000000
    .byte   %00000000
    .byte   %01000000
    .byte   %01000000
    .byte   %11111110
    .byte   %00000100
    .byte   %00000100
    .byte   %00000100
    .byte   %11111111
    .byte   %00010000
    .byte   %00000000
    .byte   %00010000
    .byte   %00000000
    .byte   %00000000
    .byte   %00000000
    .byte   0
    .byte   0
BricksCol:
    .byte   $02
    .byte   $02
    .byte   $02
    .byte   $02
    .byte   $02
    .byte   $02
    .byte   $02
    .byte   $02
    .byte   $02
    .byte   $02
    .byte   $02
    .byte   $02
    .byte   $02
    .byte   $02
    .byte   $02
    .byte   $02
    .byte   $02
    .byte   $02

PtrGraphics:
    .byte   <Prince0, <Torch0, <Gate;Bricks
PtrHi:
    .byte   >Prince0, >Torch0, >Gate;Bricks
PtrColors:
    .byte   <EnemyCol0, <TorchCol0, <GateCol;BricksCol
YPosTbl:
    .byte   KERNEL_H-FLOOR_H*3+ENEMY_H
    .byte   KERNEL_H-FLOOR_H*2+ENEMY_H+4
    .byte   KERNEL_H-FLOOR_H+ENEMY_H+6
XPosTbl:
    .byte   0
    .byte   27
    .byte   17;90
NusizTbl:
    .byte   %011101
    .byte   %000001
    .byte   %000000

    ALIGN_OFS KERNEL_H

Gate:
    .byte   %10010000
    .byte   %10010000
    .byte   %11110000
    .byte   %10010000
    .byte   %10010000
    .byte   %10010000
    .byte   %11110000
    .byte   %10010000
    .byte   %10010000
    .byte   %10010000
    .byte   %11110000
    .byte   %10010000
    .byte   %10010000
    .byte   %10010000
    .byte   %11110000
    .byte   %10010000
    .byte   %10010000

GateCol:
    .byte   $0a
    .byte   $06
    .byte   $0e
    .byte   $0c
    .byte   $0a
    .byte   $06
    .byte   $0e
    .byte   $0c
    .byte   $0a
    .byte   $06
    .byte   $0e
    .byte   $0c
    .byte   $0a
    .byte   $06
    .byte   $0e
    .byte   $0c
    .byte   $0a
    .byte   $06

GateBL:
    .byte   0, 2, 0, 2, 0, 2, 0, 2, 0, 2
    .byte   0, 2, 0, 2, 0, 2, 0, 2, 0, 2
    .byte   0, 2, 0, 2, 0, 2, 0, 2, 0, 2

LivesTbl0:
    .byte   %10001000
    .byte   %11001100
    .byte   %11101110
    .byte   %11001100
    .byte   %10001000
LivesTbl1:
    .byte   %10001000
    .byte   %01000100
    .byte   %10101010
    .byte   %01000100
    .byte   %10001000

    .byte   "Thomas Jentzsch, 2005"

    org     BASE_ADR + $ffc
    .word   Start
    .word   Start